@@ -1,29 +1,27 @@
-import json
 import os
 import tempfile
-from typing import Any, Dict, List, Tuple
+from typing import List
 
 import pytest
 from bs4 import BeautifulSoup
 
-from json_schema_for_humans.generate import generate_from_file_object, generate_from_schema, generate_from_filename
-
-
-def _get_test_case(name: str) -> Tuple[Dict[str, Any], str]:
-    """Get the loaded JSON schema for a test case"""
-    test_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "cases", f"{name}.json"))
-    with open(test_path, encoding="utf-8") as test_case_file:
-        return json.load(test_case_file), os.path.abspath(test_path)
+from json_schema_for_humans.generate import generate_from_file_object, generate_from_schema
+from tests.test_utils import _get_test_case_path
 
 
 def _generate_case(
     case_name: str, find_deprecated: bool = False, find_default: bool = False, link_to_reused_ref: bool = True
 ) -> BeautifulSoup:
     """Get the BeautifulSoup object for a test case"""
-    test_contents, test_path = _get_test_case(case_name)
     return BeautifulSoup(
         generate_from_schema(
-            test_contents, test_path, False, find_deprecated, find_default, True, link_to_reused_ref=link_to_reused_ref
+            _get_test_case_path(case_name),
+            None,
+            False,
+            find_deprecated,
+            find_default,
+            True,
+            link_to_reused_ref=link_to_reused_ref,
         ),
         "html.parser",
     )
@@ -94,10 +92,8 @@ def _assert_required(soup: BeautifulSoup, is_required_properties: List[bool]) ->
     _assert_badges(soup, "required", is_required_properties)
 
 
-def test_basic() -> None:
-    """Test rendering a basic schema with title"""
-    soup = _generate_case("basic")
-
+def _assert_basic_case(soup: BeautifulSoup) -> None:
+    """Assert the rendered result of the basic test case"""
     _assert_property_names(soup, ["firstName", "lastName", "age"])
     _assert_descriptions(
         soup,
@@ -112,6 +108,13 @@ def test_basic() -> None:
     _assert_required(soup, [False] * 3)
 
 
+def test_basic() -> None:
+    """Test rendering a basic schema with title"""
+    soup = _generate_case("basic")
+
+    _assert_basic_case(soup)
+
+
 def test_multiple_types() -> None:
     """Test rendering a schema with type being an array."""
     soup = _generate_case("multiple_types")
@@ -169,6 +172,8 @@ def test_references() -> None:
             "Description for array_def",
             "Description for string_def",
             "The delivery is a gift, no prices displayed",
+            "The delivery is a gift, no prices displayed",
+            "The delivery is a gift, no prices displayed",
             "Test schema with a not",
             "Contents of propertyA in final.json",
         ],
@@ -237,6 +242,7 @@ def test_with_multiple_descriptions():
     _assert_descriptions(
         soup,
         [
+            "Exact address",
             "Exact address",
             "Delivery info depending on the delivery type",
             "The delivery is a gift, no prices displayed",
@@ -267,7 +273,7 @@ def test_with_default() -> None:
     """Test rendering of default values"""
     soup = _generate_case("with_default")
 
-    _assert_default_values(soup, ['"Linux"', "['white', 'blue']", "2"])
+    _assert_default_values(soup, ['"Linux"', '["white", "blue"]', "2"])
 
 
 def test_deprecated_in_description() -> None:
@@ -305,13 +311,20 @@ def test_description_with_ref() -> None:
     _assert_descriptions(soup, ["We should see this", "inner description", "We should see this too"])
 
 
+def test_description_from_ref() -> None:
+    """Test that having a description next to a $ref in an object uses that description and not the one from the
+    referenced object
+    """
+    soup = _generate_case("description_from_ref")
+
+    _assert_descriptions(soup, ["a filled string"] * 2)
+
+
 def test_description_with_ref_link_to_reused_ref() -> None:
     """Same as "test_description_with_ref", but do not allow reusing references."""
     soup = _generate_case("description_with_ref", link_to_reused_ref=False)
 
-    _assert_descriptions(
-        soup, ["We should see this", "inner description", "We should see this too", "inner description"]
-    )
+    _assert_descriptions(soup, ["We should see this", "inner description", "We should see this too"])
 
 
 def test_with_examples() -> None:
@@ -341,7 +354,7 @@ def test_recursive(link_to_reused_ref: bool) -> None:
     """Test a schema having a recursive definition"""
     soup = _generate_case("recursive", link_to_reused_ref=link_to_reused_ref)
 
-    _assert_descriptions(soup, ["A human being", "The children they had"])
+    _assert_descriptions(soup, ["A human being", "The children they had", "A human being"])
 
     recursive_definition_link = soup.find("a", href="#person")
     assert recursive_definition_link
@@ -353,11 +366,11 @@ def test_recursive_array(link_to_reused_ref: bool) -> None:
     """Test a schema having a recursive definition pointing to array items"""
     soup = _generate_case("recursive_array", link_to_reused_ref=link_to_reused_ref)
 
-    _assert_descriptions(soup, ["A list of people", "A human being", "The children they had"])
+    _assert_descriptions(soup, ["A list of people", "A human being", "The children they had", "A human being"])
 
-    recursive_definition_link = soup.find("a", href="#person_array_items")
+    recursive_definition_link = soup.find("a", href="#person_items")
     assert recursive_definition_link
-    assert recursive_definition_link.text == "Same definition as person_array_items"
+    assert recursive_definition_link.text == "Same definition as person_items"
 
 
 def test_pattern_properties() -> None:
